package com.aliyun.iot.api.common.deviceApi;

import com.alibaba.fastjson.JSON;
import com.aliyun.alink.dm.api.DeviceInfo;
import com.aliyun.alink.dm.api.InitResult;
import com.aliyun.alink.linkkit.api.ILinkKitConnectListener;
import com.aliyun.alink.linkkit.api.IoTMqttClientConfig;
import com.aliyun.alink.linkkit.api.LinkKit;
import com.aliyun.alink.linkkit.api.LinkKitInitParams;
import com.aliyun.alink.linksdk.tmp.api.OutputParams;
import com.aliyun.alink.linksdk.tmp.device.payload.ValueWrapper;
import com.aliyun.alink.linksdk.tmp.devicemodel.Event;
import com.aliyun.alink.linksdk.tmp.devicemodel.Property;
import com.aliyun.alink.linksdk.tmp.listener.IPublishResourceListener;
import com.aliyun.alink.linksdk.tools.AError;
import com.aliyun.alink.linksdk.tools.ALog;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.aliyun.alink.linksdk.tools.ALog.LEVEL_DEBUG;

/**
 *  设备端代码开发
 */
public class ThingTemplate {
    private static final String TAG = "LampDemo";

    private final static String SERVICE_SET = "set";
    private final static String SERVICE_GET = "get";

    public static String DESIRED_PROPERTY_GET = "/sys/{productKey}/{deviceName}/thing/property/desired/get";
    public static String DESIRED_PROPERTY_GET_REPLY = "/sys/{productKey}/{deviceName}/thing/property/desired/get_reply";
    public static String METHOD_GET_DESIRED_PROPERTY = "thing.property.desired.get";

    public static void main(String[] args) {
        /* TODO: 替换为自己设备的证书信息 */
        String productKey = "a1qck***";
        String deviceName = "Tempdevice";
        String deviceSecret = "ffe19f***485d4e3";

        /*
        TODO: 替换为您物联网平台实例的接入地址

         对于企业版实例户新版公共实例，url的格式为"${YourInstanceId}.mqtt.iothub.aliyuncs.com"
         其中${YourInstanceId}: 请替换为您实例的Id。
         对于旧版公共实例，需要将url修改为: url = "${YourProductKey}.iot-as-mqtt.${YourRegionId}.aliyuncs.com"
         其中, ${YourProductKey}：请替换为设备所属产品的ProductKey。可登录物联网平台控制台，在对应实例的设备详情页获取。
         ${YourRegionId}：请替换为您的物联网平台设备所在地域代码, 例如cn-shanghai等
         该情况下完整url示例: a1qck***.iot-as-mqtt.cn-shanghai.aliyuncs.com
        */
        String  url = "a1qck***.iot-as-mqtt.cn-shanghai.aliyuncs.com:1883";

        /**
         * mqtt连接信息
         */
        ThingTemplate manager = new ThingTemplate();

        DeviceInfo deviceInfo = new DeviceInfo();
        deviceInfo.productKey = productKey;
        deviceInfo.deviceName = deviceName;
        deviceInfo.deviceSecret = deviceSecret;

        /**
         * 服务器端的java http 客户端使用TSLv1.2。
         */
        System.setProperty("https.protocols", "TLSv2");
        manager.init(deviceInfo, url);
    }

    public void init(final DeviceInfo deviceInfo, String url) {
        LinkKitInitParams params = new LinkKitInitParams();
        /**
         * 设置 Mqtt 初始化参数
         */
        IoTMqttClientConfig config = new IoTMqttClientConfig();
        config.productKey = deviceInfo.productKey;
        config.deviceName = deviceInfo.deviceName;
        config.deviceSecret = deviceInfo.deviceSecret;
        config.channelHost = url;

        /**
         * 是否接受离线消息
         * 对应 mqtt 的 cleanSession 字段
         */
        config.receiveOfflineMsg = false;
        params.mqttClientConfig = config;
        ALog.setLevel(LEVEL_DEBUG);
        ALog.i(TAG, "mqtt connetcion info=" + params);

        /**
         * 设置初始化，传入设备证书信息
         */
        params.deviceInfo = deviceInfo;

        /**建立链接**/
        LinkKit.getInstance().init(params, new ILinkKitConnectListener() {
            public void onError(AError aError) {
                ALog.e(TAG, "Init Error error=" + aError);
            }

            public void onInitDone(InitResult initResult) {
                ALog.i(TAG, "onInitDone result=" + initResult);

                List<Property> properties =   LinkKit.getInstance().getDeviceThing().getProperties();

                ALog.i(TAG, "设备属性列表" + JSON.toJSONString(properties));

                List<Event> getEvents  =   LinkKit.getInstance().getDeviceThing().getEvents();

                ALog.i(TAG, "设备事件列表" + JSON.toJSONString(getEvents));

                /* 属性上报. TODO: 需确保所报的属性, 比如MicSwitch, 是产品的物模型的一部分. 否则会返回错误 */
                handlePropertySet("MicSwitch", new ValueWrapper.IntValueWrapper(1));

                /* 事件上报. TODO: 需确保所报的事件, 比如Offline_alarm, 是产品的物模型的一部分. 否则会返回错误 */
                Map<String,ValueWrapper> values = new HashMap<>();
                values.put("eventValue",new ValueWrapper.IntValueWrapper(0));
                OutputParams outputParams = new OutputParams(values);
                handleEventSet("Offline_alarm",outputParams);
            }
        });
    }

    /**
     *  Alink JSON 方式设备端上报属性
     * @param identifier  属性标识符
     * @param value       上报属性值
     * @return
     */
    private void handlePropertySet(String identifier, ValueWrapper value ) {
        ALog.i(TAG, "上报 属性identity=" + identifier);

        Map<String, ValueWrapper> reportData = new HashMap<>();
        reportData.put(identifier, value);

        LinkKit.getInstance().getDeviceThing().thingPropertyPost(reportData, new IPublishResourceListener() {

            public void onSuccess(String s, Object o) {
                // 属性上报成功
                ALog.i(TAG, "上报成功 onSuccess() called with: s = [" + s + "], o = [" + o + "]");
            }

            public void onError(String s, AError aError) {
                // 属性上报失败
                ALog.i(TAG, "上报失败onError() called with: s = [" + s + "], aError = [" + JSON.toJSONString(aError) + "]");
            }
        });
    }


    /**
     *  Alink JSON 方式设备端上报事件
     * @param identifyID  事件标识符
     * @param params      事件上报参数
     * @return
     */
    private void handleEventSet(String identifyID, OutputParams params ) {
        ALog.i(TAG, "上报事件 identifyID=" + identifyID + "  params=" + JSON.toJSONString(params));


        LinkKit.getInstance().getDeviceThing().thingEventPost( identifyID,  params, new IPublishResourceListener() {

            public void onSuccess(String s, Object o) {
                // 属性上报成功
                ALog.i(TAG, "上报成功 onSuccess() called with: s = [" + s + "], o = [" + o + "]");
            }

            public void onError(String s, AError aError) {
                // 属性上报失败
                ALog.i(TAG, "上报失败onError() called with: s = [" + s + "], aError = [" + JSON.toJSONString(aError) + "]");
            }
        });
    }







}
