package com.aliyun.iot.api.common.openApi;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.iot.util.LogUtil;
import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.exceptions.ClientException;

import java.util.HashMap;
import java.util.Map;

/**
 *   从云端设置设备属性
 */
public class ThingManager extends AbstractManager {

    private static DefaultAcsClient client = getClient();

    /**
     *  设置设备的属性
     *
     */
    public static JSONObject SetDeviceProperty(String IotId, String ProductKey, String DeviceName, Map<String,Object> Items) {
        CommonRequest request = executeTests("SetDeviceProperty");
        request.putQueryParameter("IotId", IotId);
        request.putQueryParameter("ProductKey", ProductKey);
        request.putQueryParameter("DeviceName", DeviceName);
        request.putQueryParameter("Items", JSONObject.toJSONString(Items));

        CommonResponse response = null;

        try {
            response = client.getCommonResponse(request);
            JSONObject object = JSONObject.parseObject(response.getData());

            if (object.getBoolean("Success") != null && object.getBoolean("Success")) {
                LogUtil.print("下行设置设备属性成功");
                LogUtil.print(JSON.toJSONString(object));
            } else {
                LogUtil.print("下行设置设备属性失败");
                LogUtil.error(JSON.toJSONString(object));
            }
            return object.getJSONObject("Data");

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("下行设置设备属性失败！" + JSON.toJSONString(response.getData()));
        }
        return null;
    }

    /**
     *  批量设置设备属性
     *
     */
    public static JSONObject SetDevicesProperty(String ProductKey , String  DeviceName1, String  DeviceName2, Map<String,Object>  Items) {
        CommonRequest request = executeTests("SetDevicesProperty");
        request.putQueryParameter("ProductKey", ProductKey);
        request.putBodyParameter("DeviceName.1", DeviceName1);
        request.putBodyParameter("DeviceName.2", DeviceName2);
        request.putQueryParameter("Items", JSONObject.toJSONString(Items));

        CommonResponse response = null;

        try {
            response = client.getCommonResponse(request);
            JSONObject object = JSONObject.parseObject(response.getData());

            if (object.getBoolean("Success") != null && object.getBoolean("Success")) {
                LogUtil.print("下行设置设备属性成功");
                LogUtil.print(JSON.toJSONString(object));
            } else {
                LogUtil.print("下行设置设备属性失败");
                LogUtil.error(JSON.toJSONString(object));
            }
            return object.getJSONObject("Data");

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("下行设置设备属性失败！" + JSON.toJSONString(response.getData()));
        }
        return null;
    }


    /**
     *  调用设备的服务
     *
     */
    public static JSONObject InvokeThingService(String IotId,String  ProductKey,String  DeviceName,String  Identifier,String Args) {

        CommonRequest request = executeTests("InvokeThingService");
        request.putQueryParameter("IotId", IotId);
        request.putQueryParameter("ProductKey", ProductKey);
        request.putQueryParameter("DeviceName", DeviceName);
        request.putQueryParameter("Identifier", Identifier);
        request.putQueryParameter("Args", Args);
        CommonResponse response = null;

        try {
            response = client.getCommonResponse(request);
            JSONObject object = JSONObject.parseObject(response.getData());

            if (object.getBoolean("Success") != null && object.getBoolean("Success")) {
                LogUtil.print("调用设备的服务成功");
                LogUtil.print(JSON.toJSONString(object));
            } else {
                LogUtil.print("调用设备的服务失败");
                LogUtil.error(JSON.toJSONString(object));
            }
            return object.getJSONObject("Data");

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("下行设置设备属性失败！" + JSON.toJSONString(response.getData()));
        }
        return null;
    }


    /**
     *  批量调用设备的服务
     *
     */
    public static JSONObject InvokeThingsService(String  ProductKey,String  name1,String name2,String  Identifier,String  Args) {
        CommonRequest request = executeTests("InvokeThingsService");
        request.putQueryParameter("ProductKey", ProductKey);
        request.putBodyParameter("DeviceName.1", name1);
        request.putBodyParameter("DeviceName.2", name2);
        request.putQueryParameter("Identifier", Identifier);
        request.putQueryParameter("Args", Args);
        CommonResponse response = null;

        try {
            response = client.getCommonResponse(request);
            JSONObject object = JSONObject.parseObject(response.getData());

            if (object.getBoolean("Success") != null && object.getBoolean("Success")) {
                LogUtil.print("批量调用设备的服务成功");
                LogUtil.print(JSON.toJSONString(object));
            } else {
                LogUtil.print("批量调用设备的服务失败");
                LogUtil.error(JSON.toJSONString(object));
            }
            return object.getJSONObject("Data");

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("批量调用设备的服务失败！" + JSON.toJSONString(response.getData()));
        }
        return null;
    }


    public static void main(String[] args) {
        //1 设置设备的属性
        Map<String,Object> propertymap =new HashMap<>();
        propertymap.put("WindSpeed_10",0);

        SetDeviceProperty("","a1BxptK****","safasda",propertymap);

        //2 批量设置设备属性


        Map<String,Object> propertymapNames =new HashMap<>();
        propertymapNames.put("hue",0);

        SetDevicesProperty("a1usVke****","C08ASLugvCzmqkfa****","gM8e7ew22be5dZCP****",propertymapNames);

        //3 调用设备的服务
        InvokeThingService("C08ASLugvCzmqkfaJ****000100",null,null,"ModifyVehicleInfo","{}");

        //4 批量调用设备的服务
        InvokeThingsService("a1usVkeS****","C08ASLugvCzmqkfa****","gM8e7ew22be5dZCP****","ModifyVehicleInfo","{}");

    }

}
