package com.aliyun.iot.api.common.openApi;

import com.alibaba.fastjson.JSON;
import com.aliyun.iot.api.common.openApi.AbstractManager;
import com.aliyun.iot.util.LogUtil;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.iot.model.v20180120.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ThingManagerForPopSDk  extends AbstractManager {

    private static DefaultAcsClient client = getClient();

    /**
     * 设置设备的属性
     */
    public static void SetDeviceProperty(String InstanceId, String IotId, String ProductKey, String DeviceName , String Items) {
        SetDevicePropertyResponse response =null;
        SetDevicePropertyRequest request=new SetDevicePropertyRequest();
        request.setDeviceName(DeviceName);
        request.setIotId(IotId);
        request.setItems(Items);
        request.setProductKey(ProductKey);
        request.setIotInstanceId(InstanceId);

        try {
            response = client.getAcsResponse(request);

            if (response.getSuccess() != null && response.getSuccess()) {
                LogUtil.print("设置设备的属性成功");
                LogUtil.print(JSON.toJSONString(response));
            } else {
                LogUtil.print("设置设备的属性失败");
                LogUtil.error(JSON.toJSONString(response));
            }

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("设置设备的属性失败！" + JSON.toJSONString(response));
        }
    }

    /**
     * 批量设置设备属性
     *
     * @param ProductKey      设置属性的设备所隶属的产品Key
     * @param DeviceNames     要设置属性的设备名称列表。
     * @param Items  要设置的属性信息，组成为key:value，数据格式为 JSON String。  必须
     *
     * @Des 描述：
     */
    public static void SetDevicesProperty(String InstanceId, String ProductKey, List<String> DeviceNames, String Items) {
        SetDevicesPropertyResponse response = new SetDevicesPropertyResponse();
        SetDevicesPropertyRequest request = new SetDevicesPropertyRequest();
        request.setDeviceNames(DeviceNames);
        request.setItems(Items);
        request.setProductKey(ProductKey);
        request.setIotInstanceId(InstanceId);

        try {
            response = client.getAcsResponse(request);

            if (response.getSuccess() != null && response.getSuccess()) {
                LogUtil.print("批量设置设备属性成功");
                LogUtil.print(JSON.toJSONString(response));
            } else {
                LogUtil.print("批量设置设备属性失败");
                LogUtil.error(JSON.toJSONString(response));
            }
        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("批量设置设备属性失败！" + JSON.toJSONString(response));
        }
    }


    /**
     * 在一个设备上执行指定的服务（调用设备服务）
     *
     * @param IotId      要查询的设备ID
     * @param ProductKey 设备所隶属的产品Key
     * @param DeviceName 设备的名称
     * @param Identifier 高级版设备的服务Identifier  必须
     * @param Args       要启用服务的入参信息    必须
     * @Des 描述：
     */
    public static InvokeThingServiceResponse.Data InvokeThingService(String InstanceId, String IotId, String ProductKey, String DeviceName,
                                                                     String Identifier, String Args) {
        InvokeThingServiceResponse response =null;
        InvokeThingServiceRequest request = new InvokeThingServiceRequest();
        request.setArgs(Args);
        request.setDeviceName(DeviceName);
        request.setIotId(IotId);
        request.setIdentifier(Identifier);
        request.setProductKey(ProductKey);
        request.setIotInstanceId(InstanceId);

        try {
            response = client.getAcsResponse(request);

            if (response.getSuccess() != null && response.getSuccess()) {
                LogUtil.print("服务执行成功");
                LogUtil.print(JSON.toJSONString(response));
            } else {
                LogUtil.print("服务执行失败");
                LogUtil.error(JSON.toJSONString(response));
            }
            return response.getData();

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("服务执行失败！" + JSON.toJSONString(response));
        }
        return null;
    }

    /**
     * 批量调用设备服务
     *
     * @param IotId      要查询的设备ID  必须
     * @param ProductKey 设备所隶属的产品Key  必须
     * @param DeviceNames 设备的名称列表。必须
     * @param Identifier  高级版设备的服务Identifier  必须
     * @param Args 要启用服务的入参信息  必须
     *
     * @Des 描述：
     */
    public static void InvokeThingsService(String InstanceId, String IotId, String ProductKey, List<String> DeviceNames,
                                           String Identifier, String Args) {
        InvokeThingsServiceResponse response =null;
        InvokeThingsServiceRequest request = new InvokeThingsServiceRequest();
        request.setArgs(Args);
        request.setIdentifier(Identifier);
        request.setDeviceNames(DeviceNames);
        request.setProductKey(ProductKey);
        request.setIotInstanceId(InstanceId);


        try {
            response = client.getAcsResponse(request);

            if (response.getSuccess() != null && response.getSuccess()) {
                LogUtil.print("批量调用设备服务成功");
                LogUtil.print(JSON.toJSONString(response));
            } else {
                LogUtil.print("批量调用设备服务失败");
                LogUtil.error(JSON.toJSONString(response));
            }

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("批量调用设备服务失败！" + JSON.toJSONString(response));
        }
    }


    public static void main(String[] args) {

        /**上线设备*/
        String deviceName = "Tempdevice";
        String deviceProductkey = "a1qck***";

        /**对于企业版实例和新版公共实例，设置InstanceId值为具体实例ID值，您可在物联网平台控制台的实例概览页面，查看当前实例的ID。
         *对于旧版公共实例,设置InstanceId值为空值""。
         */
        String InstanceId = "";


        Map<String, Object> ext =  new HashMap<>();
        ext.put("hue", 0);

        //1 设置设备的属性
        SetDeviceProperty(InstanceId,null, deviceProductkey, deviceName, JSON.toJSONString(ext));

        //2 批量设置设备属性
        List<String>  deviceNames = new ArrayList<>();
        deviceNames.add(deviceName);

        SetDevicesProperty(InstanceId,deviceProductkey, deviceNames, JSON.toJSONString(ext));

        //3 调用设备的服务
        InvokeThingService(InstanceId,null, deviceProductkey, deviceName, "ModifyVehicleInfo", "{}");

        //4 批量调用设备的服务
        List<String>  deviceNamesService = new ArrayList<>();
        deviceNamesService.add(deviceName);
        InvokeThingsService(InstanceId,null, deviceProductkey, deviceNamesService, "ModifyVehicleInfo", "{}");

    }
}
