package com.aliyun.iot.api.common.openApi;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.iot.util.LogUtil;
import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.exceptions.ClientException;

import java.util.HashMap;
import java.util.Map;


/**
 *  从云端添加设备topo关系
 */

public class TopoManager  extends AbstractManager  {


    private static DefaultAcsClient client = getClient();

    /**
     *  通知云端增加设备拓扑关系
     */
    public static JSONObject NotifyAddThingTopo(String GwIotId, String GwProductKey, String GwDeviceName, JSONArray DeviceListStr) {
        CommonRequest request = executeTests("NotifyAddThingTopo");
        request.putQueryParameter("GwIotId", GwIotId);
        request.putQueryParameter("GwProductKey", GwProductKey);
        request.putQueryParameter("GwDeviceName", GwDeviceName);
        request.putQueryParameter("DeviceListStr", JSONObject.toJSONString(DeviceListStr));

        CommonResponse response = null;

        try {
            response = client.getCommonResponse(request);
            JSONObject object = JSONObject.parseObject(response.getData());

            if (object.getBoolean("Success") != null && object.getBoolean("Success")) {
                LogUtil.print("增加设备拓扑关系成功");
                LogUtil.print(JSON.toJSONString(object));
            } else {
                LogUtil.print("增加设备拓扑关系失败");
                LogUtil.error(JSON.toJSONString(object));
            }
            return object.getJSONObject("Data");

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("增加设备拓扑关系失败！" + JSON.toJSONString(response.getData()));
        }
        return null;
    }

    /**
     *  查询网关设备或子设备所具有的拓扑关系
     */
    public static JSONObject GetThingTopo(String IotId, String ProductKey, String DeviceName,Integer PageSize , Integer PageNo) {
        CommonRequest request = executeTests("GetThingTopo");
        request.putQueryParameter("IotId", IotId);
        request.putQueryParameter("ProductKey", ProductKey);
        request.putQueryParameter("DeviceName", DeviceName);
        request.putBodyParameter("PageSize", PageSize);
        request.putBodyParameter("PageNo", PageNo);

        CommonResponse response = null;

        try {
            response = client.getCommonResponse(request);
            JSONObject object = JSONObject.parseObject(response.getData());

            if (object.getBoolean("Success") != null && object.getBoolean("Success")) {
                LogUtil.print("查询设备拓扑关系成功");
                LogUtil.print(JSON.toJSONString(object));
            } else {
                LogUtil.print("查询设备拓扑关系失败");
                LogUtil.error(JSON.toJSONString(object));
            }
            return object.getJSONObject("Data");

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("查询设备拓扑关系失败！" + JSON.toJSONString(response.getData()));
        }
        return null;
    }


    /**
     *  移除网关设备或子设备所具有的拓扑关系
     */
    public static JSONObject RemoveThingTopo(String IotId, String ProductKey, String DeviceName) {
        CommonRequest request = executeTests("RemoveThingTopo");
        request.putQueryParameter("IotId", IotId);
        request.putQueryParameter("ProductKey", ProductKey);
        request.putQueryParameter("DeviceName", DeviceName);

        CommonResponse response = null;

        try {
            response = client.getCommonResponse(request);
            JSONObject object = JSONObject.parseObject(response.getData());

            if (object.getBoolean("Success") != null && object.getBoolean("Success")) {
                LogUtil.print("移除设备拓扑关系成功");
                LogUtil.print(JSON.toJSONString(object));
            } else {
                LogUtil.print("移除设备拓扑关系失败");
                LogUtil.error(JSON.toJSONString(object));
            }
            return object.getJSONObject("Data");

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("移除设备拓扑关系失败！" + JSON.toJSONString(response.getData()));
        }
        return null;
    }



    /**
     *  移除网关设备或子设备所具有的拓扑关系
     */
    public static JSONObject RegisterDevice(String ProductKey, String DeviceName) {
        CommonRequest request = executeTests("RegisterDevice");
        request.putQueryParameter("ProductKey", ProductKey);
        request.putQueryParameter("DeviceName", DeviceName);

        CommonResponse response = null;

        try {
            response = client.getCommonResponse(request);
            JSONObject object = JSONObject.parseObject(response.getData());

            if (object.getBoolean("Success") != null && object.getBoolean("Success")) {
                LogUtil.print("注册设备成功");
                LogUtil.print(JSON.toJSONString(object));
            } else {
                LogUtil.print("注册设备失败");
                LogUtil.error(JSON.toJSONString(object));
            }
            return object.getJSONObject("Data");

        } catch (ClientException e) {
            e.printStackTrace();
            LogUtil.error("注册设备失败！" + JSON.toJSONString(response.getData()));
        }
        return null;
    }


    public static void main(String[] args) {
        //子设备动态注册获取设备deviceSecret，如果设备已知设备证书则忽略此步
        //预注册设备时，可以使用设备的MAC地址或SN序列号等作为DeviceName
        JSONObject jsonObject = RegisterDevice("a12Z3Lf****","CtvfOzrTv9nBSOwc59HW4");

        //网关查询当前子设备是否与自己有topo关系
        JSONObject topoList = GetThingTopo("","a12Z3Lf****","CtvfOzrTv9nBSOwc59HW4",1,100);

        if(topoList.get("Data")!=null){
            LogUtil.error("设备已经绑定topo关系");
            return;
        }

        //通知云端增加设备拓扑关系
        JSONArray topoArray = new JSONArray();
        JSONObject object = new JSONObject();
        object.put("ProductKey","a12Z3Lf****");
        object.put("DeviceName","CtvfOzrTv9nBSOwc59HW4");
        topoArray.add(object);

        NotifyAddThingTopo("","a1BxptK****","NYgA08M3DLS81sHRA6DQ",topoArray);

        // 查询网关设备或子设备所具有的拓扑关系
        GetThingTopo("","a1D8fJX****","0S10yKCel4LGht7K91VV",1,10);

        // 移除网关设备或子设备所具有的拓扑关系
        //RemoveThingTopo("","","");

    }
}
